@@ -423,6 +423,13 @@ static void do_keep_aspect_ratio(GtkWidget *menu, GtkWidget *vncdisplay)
         vnc_display_set_keep_aspect_ratio(VNC_DISPLAY(vncdisplay), FALSE);
 }
 
+static void do_rotate(GtkWidget *menu G_GNUC_UNUSED, GtkWidget *vncdisplay)
+{
+    guint rotation = vnc_display_get_rotation(VNC_DISPLAY(vncdisplay));
+    vnc_display_set_rotation(VNC_DISPLAY(vncdisplay), rotation + 90u);
+}
+
+
 static void do_power_control(VncDisplay *vncdisplay, VncConnectionPowerAction action)
 {
     VncConnection *conn = vnc_display_get_connection(vncdisplay);
@@ -721,6 +728,7 @@ int main(int argc, char **argv)
     GtkWidget *smoothing;
     GtkWidget *keep_aspect_ratio;
     GtkWidget *resize;
+    GtkWidget *rotate;
     GtkWidget *showgrabkeydlg;
     GtkWidget *shutdown;
     GtkWidget *reboot;
@@ -800,6 +808,7 @@ int main(int argc, char **argv)
     smoothing = gtk_check_menu_item_new_with_mnemonic("Smooth scaling");
     keep_aspect_ratio = gtk_check_menu_item_new_with_mnemonic("Keep aspect ratio");
     resize = gtk_check_menu_item_new_with_mnemonic("Allow resizing");
+    rotate = gtk_menu_item_new_with_mnemonic("Rotate ↷");
 
     gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(scaling), TRUE);
     gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(smoothing), TRUE);
@@ -809,6 +818,7 @@ int main(int argc, char **argv)
     gtk_menu_shell_append(GTK_MENU_SHELL(submenu), scaling);
     gtk_menu_shell_append(GTK_MENU_SHELL(submenu), smoothing);
     gtk_menu_shell_append(GTK_MENU_SHELL(submenu), keep_aspect_ratio);
+    gtk_menu_shell_append(GTK_MENU_SHELL(submenu), rotate);
     gtk_menu_shell_append(GTK_MENU_SHELL(submenu), resize);
 
     gtk_menu_item_set_submenu(GTK_MENU_ITEM(view), submenu);
@@ -892,6 +902,7 @@ int main(int argc, char **argv)
 
     vnc_display_set_scaling(VNC_DISPLAY(vnc), TRUE);
     vnc_display_set_allow_resize(VNC_DISPLAY(vnc), TRUE);
+    vnc_display_set_rotation(VNC_DISPLAY(vnc), 0u);
     vnc_display_set_lossy_encoding(VNC_DISPLAY(vnc), TRUE);
     vnc_display_set_zoom_level(VNC_DISPLAY(vnc), opt_zoom);
 
@@ -963,6 +974,8 @@ int main(int argc, char **argv)
                      G_CALLBACK(do_smoothing), vnc);
     g_signal_connect(keep_aspect_ratio, "toggled",
                      G_CALLBACK(do_keep_aspect_ratio), vnc);
+    g_signal_connect(rotate, "activate",
+                     G_CALLBACK(do_rotate), vnc);
     g_signal_connect(shutdown, "activate",
                      G_CALLBACK(do_shutdown), vnc);
     g_signal_connect(reboot, "activate",
